package {
	import flash.desktop.Clipboard;
	import flash.desktop.ClipboardFormats;
	import flash.desktop.NativeApplication;
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.InvokeEvent;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Matrix;
	import flash.geom.Rectangle;
	import flash.utils.Timer;
	
	import mx.core.BitmapAsset;
	
	public class IconManager extends EventDispatcher 
	{
		[Embed(source="assets/translator_icon16.png")]
		private var icon16:Class;
		[Embed(source="assets/translator_icon128.png")]
		private var icon128:Class;
		private var newIconBitmap:BitmapData;           // アイコンくるくる用
		
		private var degrees:int;
		private var scales:int;
		private var edge:int;
		
		private var isAnimate:Boolean = false;
		private var isSecond:Boolean = false;
		private var isDecrement:Boolean = false;
		private var isReverse:Boolean = false;
		
		private var _gmodel:TranslateModel;
		private var _timer:Timer;
		
		public function IconManager():void {
		}
		
		public function initIconManager(gmodel:TranslateModel):void {
			_gmodel = gmodel;
			
			_timer = new Timer(10000);
			_timer.addEventListener(TimerEvent.TIMER, onTimerHandler);
			
			initIcon();
		}
		
		private function initIcon():void {
			if (NativeApplication.supportsSystemTrayIcon) {
				// タスクトレイアイコン
				newIconBitmap = (new icon16() as BitmapAsset).bitmapData;
				NativeApplication.nativeApplication.icon.addEventListener(MouseEvent.CLICK, onIconClick);
				isSecond = true;
				edge = 16;
			}
			else if (NativeApplication.supportsDockIcon) {
				// ドックアイコン
				newIconBitmap = (new icon128() as BitmapAsset).bitmapData;
				NativeApplication.nativeApplication.addEventListener(InvokeEvent.INVOKE, onIconClick);
				edge = 128;
			}
			else {
				NativeApplication.nativeApplication.exit();
			}
			
			setIcon(newIconBitmap);
		}
		
		private function setIcon(bitmap:BitmapData):void {
			NativeApplication.nativeApplication.icon.bitmaps = [bitmap];
		}
		
		private function onIconClick(event:Event):void {
			if (isSecond) {
				if (isAnimate) {
					if (NativeApplication.supportsSystemTrayIcon) {
						setIcon(newIconBitmap);
					}
					
					isAnimate = false;
				}
				else {
					if (!Clipboard.generalClipboard.hasFormat(ClipboardFormats.TEXT_FORMAT)) {
						return;
					}
					
					_gmodel.Original = Clipboard.generalClipboard.getData(ClipboardFormats.TEXT_FORMAT) as String;
				}
				
				dispatchEvent(new TranslatorSampleEvent(TranslatorSampleEvent.ICON_CLICK));
			}
			else {
				isSecond = true;
			}
		}
		
		public function iconAnimate(event:Event):void {
			var iconBitmap:BitmapData;
			var newIconBitmap:BitmapData;
			var rectangle:Rectangle;
			
			isAnimate = true;
			degrees++;
			
			if (isDecrement) {
				scales--;
			}
			else {
				scales++;
			}
			
			iconBitmap = new BitmapData(edge, edge, true, 0x00000000);
			newIconBitmap = (((edge == 128) ? new icon128() : new icon16()) as BitmapAsset).bitmapData;
			rectangle = new Rectangle(0, 0, edge, edge);
			
			setIcon(moveBitmap(iconBitmap, newIconBitmap, rectangle, scales, degrees, isReverse));
			
			if (scales == edge || scales == 0) {
				if (scales == edge) {
					isDecrement = true;
					isReverse = !isReverse;
				}
				else if (scales == 0) {
					isDecrement = false;
				}
			}
			
			if (degrees == 360) {
				degrees = 0;
			}
		}
		
		private function moveBitmap(iconBitmap:BitmapData, newIconBitmap:BitmapData, 
			rectangle:Rectangle, scales:int, degrees:int, isReverse:Boolean):BitmapData {
			var matrixRotate:Matrix;
			var matrixScaling:Matrix;
			var radian:Number = degrees/180*Math.PI;
			
			if (isReverse) {
				var tempBitmap:BitmapData = new BitmapData(edge, edge, true, 0x00000000);
				var tempMatrix:Matrix = new Matrix();
				var tempRectangle:Rectangle = new Rectangle(0, 0, edge, edge);
				tempMatrix.scale(-1, 1);
				tempMatrix.translate(edge, 0);
				tempBitmap.draw(newIconBitmap, tempMatrix, null, null, tempRectangle);
				newIconBitmap = tempBitmap.clone();
			}
			
			matrixScaling = new Matrix();
			var sx:Number;
			var sy:Number;
			var tx:Number;
			var ty:Number;
			sx = (edge-scales)/edge;
			sy = 1;
			tx = edge/2-(sx*edge/2);
			ty = 0; 
			matrixScaling.scale(sx, sy);
			matrixScaling.translate(tx, ty);
			
			matrixRotate = new Matrix();
			matrixRotate.rotate(radian);
			matrixRotate.translate(edge/2-(Math.cos(radian)*edge/2-Math.sin(radian)*edge/2),
				edge/2-(Math.sin(radian)*edge/2+Math.cos(radian)*edge/2));

			matrixScaling.concat(matrixRotate);
			iconBitmap.draw(newIconBitmap, matrixScaling, null, null, rectangle);
			
			return iconBitmap;
		}
		
		private function onTimerHandler(event:TimerEvent):void {
			isAnimate = false;
			
			if (NativeApplication.supportsSystemTrayIcon) {
				setIcon(newIconBitmap);
			}
			
			dispatchEvent(new TranslatorSampleEvent(TranslatorSampleEvent.ICON_TIMER));
		}
		
		public function timerStart():void {
			_timer.start();
		}
		
		public function timerStop():void {
			_timer.stop();
		}
		
	}
}