package
{
	import com.adobe.serialization.json.JSON;
	import com.hurlant.crypto.Crypto;
	import com.hurlant.crypto.hash.HMAC;
	import com.hurlant.util.Base64;
	import com.hurlant.util.Hex;
	
	import flash.net.URLVariables;
	import flash.utils.ByteArray;
	
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.HTTPService;
	import mx.utils.UIDUtil;
	
	public class TwitterManager
	{
		private var _model:XAuthModel;
		
		public function TwitterManager()
		{
		}
		
		public function initTwitterManager(model:XAuthModel):void {
			_model = model;
		}
		
		public function Tweet(message:String):void {
			if (!_model.isAuthorized) {
				getAuthToken(message);
			}
			else {
				setStatus(message);
			}
		}

		private function getAuthToken(status:String):void {
			_model.Status = status;
			
			var requestTokenService:HTTPService = new HTTPService();
			requestTokenService.method = "POST";
			requestTokenService.url = _model.AccessTokenUrl;
			requestTokenService.addEventListener(ResultEvent.RESULT, onAuthTokenResult);
			requestTokenService.addEventListener(FaultEvent.FAULT, onAuthTokenFault);
			
			var forms:URLVariables = new URLVariables();
			forms.x_auth_mode = "client_auth";
			forms.x_auth_password = "password";
			forms.x_auth_username = "username";
			forms.oauth_signature = getSignature(requestTokenService.method, requestTokenService.url, forms);
			
 			requestTokenService.request = forms;
			requestTokenService.send();
		}
		
		private function getSignature(method:String, url:String, forms:URLVariables):String {
			var now:Date = new Date();
			
			forms.oauth_consumer_key = _model.ConsumerKey;
			forms.oauth_nonce = UIDUtil.getUID(now);
			forms.oauth_signature_method = "HMAC-SHA1";
			forms.oauth_timestamp = now.time.toString().substring(0, 10);
			forms.oauth_version = "1.0";
			if (_model.OAuthToken.length) {
				forms.oauth_token = _model.OAuthToken; 
			}
			
			var sortArr:Array = UrlVariableToArray(forms);
			var sigBase:String = URLEncoding.encode(method) + "&" +  URLEncoding.encode(url)  + "&" + URLEncoding.encode(sortArr.join("&"));
			var sigkeybase:String = URLEncoding.encode(_model.ConsumerSecret) + "&";
			if (_model.OAuthTokenSecret.length){
				sigkeybase += URLEncoding.encode(_model.OAuthTokenSecret);
			}
			
			var hmac:HMAC = Crypto.getHMAC("sha1");
			var sig_key:ByteArray = Hex.toArray(Hex.fromString(sigkeybase));
			
			var data:ByteArray = Hex.toArray(Hex.fromString(sigBase));
			var signature:String = Base64.encodeByteArray(hmac.compute(sig_key, data));
				
			return signature;
		}
		
		private function UrlVariableToArray(variables:URLVariables):Array{
			var arr:Array = new Array();
			for(var key:String in variables){
				arr.push(key + "=" + URLEncoding.encode(variables[key]));
			}
			arr.sort();
			return arr;
		}
		
		private function onAuthTokenResult(event:ResultEvent):void {
			var results:Array  = event.result.toString().split("&");
			
			for each (var token:String in results) {
				if (_model.OAuthToken == "" || _model.OAuthTokenSecret == "") {
					var values:Array = token.split("=");
					
					if (values.length == 2) {
						if (values[0] == "oauth_token") {
							_model.OAuthToken = values[1];
						}
						else if (values[0] == "oauth_token_secret") {
							_model.OAuthTokenSecret = values[1];
						}
					}
				}
				else {
					break;
				}
			}
			
			_model.isAuthorized = true;
			
			setStatus(_model.Status);
		}
		
		private function onAuthTokenFault(event:FaultEvent):void {
			trace("投稿失敗...\n" + event.fault.toString());
		}
		
		private function setStatus(status:String):void {
			var updateService:HTTPService = new HTTPService;
			updateService.method = "POST";
			updateService.url = "http://twitter.com/statuses/update.json";
			updateService.resultFormat = "text";
			updateService.addEventListener(ResultEvent.RESULT, onResult);
			updateService.addEventListener(FaultEvent.FAULT, onFault);

			var forms:URLVariables = new URLVariables();
			forms.status = status;
			forms.oauth_signature = getSignature(updateService.method, updateService.url, forms);
			updateService.request = forms;
			updateService.send();
		}
		
		private function onResult(event:ResultEvent):void {
			var json:Object = JSON.decode(event.result.toString());
			trace(json.text);
		}
		
		private function onFault(event:FaultEvent):void {
			trace("投稿失敗...\n" + event.fault.toString());
		}
	}
}